-- Permission is granted to do whatever one wants with the code, and even
-- replace the contents of the original SVN module. I would be pleased if someone
-- started committing all sorts of crazy stuff.

local inCombat = nil
local dndBanner = "<BossWhisperer> "

local dndString = dndBanner .. "I'm busy fighting %s (currently at %d%%). You'll be notified when combat ends."
local combatEndedString = dndBanner .. "Combat ended after %d minutes."

do
	local locale = GetLocale()
	if locale == "deDE" then
		dndString = dndBanner .. "Ich bin im Kampf mit %s (noch %d%% Leben). Du wirst nach Kampfende benachrichtigt werden."
		combatEndedString = dndBanner .. "Kampf beendet nach %d Minuten."
	end
end

local whisperers = {}
local combatStart = nil
local boss = nil
local bossHp = nil

local pname = UnitName("player")
local orig = ChatFrame_MessageEventHandler
ChatFrame_MessageEventHandler = function(event, ...)
	if combatStart and type(arg2) == "string" then
		if event == "CHAT_MSG_WHISPER_INFORM" and arg2 ~= pname then
			-- Hide AddOns outgoing whispers
			if arg1:sub(1, dndBanner:len()) == dndBanner then
				return
			end
		elseif event == "CHAT_MSG_WHISPER" and arg2 ~= pname then
			-- If not whispering self then let him know we are fighting a boss (but don't reply to BossWhisperer2 messages)
			if arg1:sub(1, dndBanner:len()) == dndBanner then
				return orig(event, ...)
			end
			whisperers[arg2] = true
			SendChatMessage(dndString:format(boss, bossHp), "WHISPER", nil, arg2)
		end
	elseif type(arg2) == "string" then
		if event == "CHAT_MSG_WHISPER_INFORM" and arg2 ~= pname then
			-- Hide AddOns outgoing whispers
			if arg1:sub(1, dndBanner:len()) == dndBanner then
				return
			end
		end
	end
	return orig(event, ...)
end

local function checkTarget(id)
	return UnitExists(id) and UnitAffectingCombat(id) and UnitClassification(id) == "worldboss"
end

local function partied(name)
	if UnitInRaid("player") then
		for i = 1, 40 do
			if UnitName("raid"..i) == name then return true end
		end
	else
		for i = 1, 5 do
			if UnitName("party"..i) == name then return true end
		end
	end
	return false
end

local function scan()
	if checkTarget("target") then return "target" end
	if checkTarget("focus") then return "focus" end
	if UnitInRaid("player") then
		for i = 1, 40 do
			if checkTarget("raid"..i.."target") then return "raid"..i.."target" end
		end
	else
		for i = 1, 5 do
			if checkTarget("party"..i.."target") then return "party"..i.."target" end
		end
	end
end

local frame = CreateFrame("Frame")
frame:SetScript("OnEvent", function()
	if event == "PLAYER_REGEN_DISABLED" then
		local target = scan()
		if not target then return end
		combatStart = GetTime()
		boss = UnitName(target)
		bossHp = UnitHealth(target)
		frame:RegisterEvent("UNIT_HEALTH")
	elseif event == "PLAYER_REGEN_ENABLED" and combatStart then
		local time = GetTime() - combatStart
		local msg = combatEndedString:format(math.floor(time / 60))
		for k, v in pairs(whisperers) do
			-- Notify people that combat has ended
			SendChatMessage(msg, "WHISPER", nil, k)
			whisperers[k] = nil
		end
		combatStart = nil
		boss = nil
		bossHp = nil
		frame:UnregisterEvent("UNIT_HEALTH")
	elseif event == "UNIT_HEALTH" and arg1 and UnitName(arg1) == boss then
		bossHp = UnitHealth(arg1)
	end
end)

frame:RegisterEvent("UNIT_TARGET")
frame:RegisterEvent("PLAYER_REGEN_DISABLED")
frame:RegisterEvent("PLAYER_REGEN_ENABLED")

